program service_example;

{$mode objfpc}{$H+}
{$define usecthreads}
{$apptype gui}

uses
  {$IFDEF UNIX}{$IFDEF UseCThreads}
  cthreads,
  {$ENDIF}{$ENDIF}
  sysutils,
  Classes
  { add your units here }

  // fcl-extra package --
  , daemonapp
  // --------------------
  ;

type

  { TThreadWorker }

  TThreadWorker = Class(TThread)
  // does a beep every few seconds
  protected
    Procedure Execute; override;
  public
    FIsPaused : boolean ;
  end;


  { TExampleDaemon }

  TExampleDaemon = Class(TCustomDaemon)
  Private
    FThreadWorker : TThreadWorker                       ;
    Procedure HandleWorkerTerminated (Sender : TObject) ;
  public
    function ShutDown  () : Boolean ; override ;
    function Stop      () : Boolean ; override ;

    function Start     () : Boolean ; override ;

    function Pause     () : Boolean ; override ;
    function Continue  () : Boolean ; override ;
  end;

  { TExampleDaemonMapper }

  TExampleDaemonMapper = Class(TCustomDaemonMapper)
  public
    Constructor Create(AOwner : TComponent); override;
  end;

//------------------------------------------------------------------------------


procedure TThreadWorker.Execute;
const one_second = 1000 ;
begin
  // give signal that thread is working
  // beep every 3 seconds
  repeat
    if ( not FIsPaused ) then begin
      Beep                      ;
      Sleep ( 3 * one_second ) ;
    end;
  until Terminated              ;
end;

//------------------------------------------------------------------------------

procedure TExampleDaemon.HandleWorkerTerminated(Sender: TObject);
begin
  FreeAndNil ( FThreadWorker ) ;
end;

function TExampleDaemon.Start: Boolean;
begin
  Result := inherited Start                           ;

  // create & start thread process
  FThreadWorker                 := TThreadWorker.Create(True) ;
  FThreadWorker.FIsPaused       := False                      ;
  FThreadWorker.OnTerminate     := @HandleWorkerTerminated    ;
  FThreadWorker.FreeOnTerminate := False                      ;
  FThreadWorker.Start                                         ;
end;

function TExampleDaemon.Stop: Boolean;
begin
  Result := inherited Stop ;
  // terminate thread
  FThreadWorker.Terminate        ;
end;

function TExampleDaemon.Pause: Boolean;
begin
  Result                  := inherited Pause    ;
  FThreadWorker.FIsPaused := True               ;
end;

function TExampleDaemon.Continue: Boolean;
begin
  Result                  := inherited Continue ;
  FThreadWorker.FIsPaused := False              ;
end;


function TExampleDaemon.ShutDown: Boolean;
begin
  Result := inherited ShutDown ;
  // terminate thread
  FThreadWorker.Terminate            ;
end;


//------------------------------------------------------------------------------


constructor TExampleDaemonMapper.Create(AOwner: TComponent);
const s_service_name         = 'service_example'             ;
const s_service_display_name = 'service_example_displayname' ;
//const s_pascal_classname     = 'TExampleDaemon'              ;
var MyDaemonDef : TDaemonDef                                 ;
begin
  inherited Create(AOwner);
  MyDaemonDef                         := DaemonDefs.Add as TDaemonDef ;
  MyDaemonDef.DaemonClassName         := TExampleDaemon.ClassName     ;// s_pascal_classname ;
  MyDaemonDef.Name                    := s_service_name               ;
  MyDaemonDef.DisplayName             := s_service_display_name       ;
  MyDaemonDef.WinBindings.ServiceType := stWin32                      ;
end;

//------------------------------------------------------------------------------

//------------------------------------------------------------------------------
// program begin
//------------------------------------------------------------------------------
const s_application_title = 'service_example_app' ;
begin
  RegisterDaemonClass  ( TExampleDaemon       )   ;
  RegisterDaemonMapper ( TExampleDaemonMapper )   ;
  Application.Title := s_application_title        ;
  Application.Run                                 ;
end.


